// build.cc - rectification support
//
// Copyright (C) 2000 Bruce I Mills <B.I.Mills@cs.cf.ac.uk>
// Copyright (C) 2001-2002 Frank C Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.

#include "config.h"
#include "points.hxx"
#include "build.hxx"
#include "lapack.h"

extern "C" {
  int (*MAIN__) (int, char **) = NULL;
}

void qmatrix::dual (double x, double y, double z)
{
  data[0] = x;
  data[1] = y;
  data[2] = z;

  data[3] = data[4] = data[5] = data[6] = data[7] = data[8] = 0.0;

  return;
}

void qmatrix::vect (double x, double y, double z)
{
  data[0] = x; 
  data[3] = y;
  data[6] = z;

  data[1] = data[2] = data[4] = data[5] = data[7] = data[8] = 0.0;

  return;
}

void qmatrix::diag (double x, double y, double z)
{
  data[0] = x;
  data[4] = y;
  data[8] = z;

  data[1] = data[2] = data[3] = data[5] = data[6] = data[7] = 0.0;
  
  return;
}

void qmatrix::put (char *str1, char *str2)
{
  if (str1)
    cout << str1;
  for (word j = 0; j < 3; ++j) {
    for (word i = 0; i < 3; ++i) 
      printf ("%8.6f ", data[j * 3 + i]);
    cout << "\n";
  }
  if(str2)
    cout << str2;
  return;
}

qmatrix operator* (double a, qmatrix const &b)
{
  qmatrix c;
  for(word i = 0; i < 9; ++i)
    c.data[i] = a * b.data[i];
  return c;
}

qmatrix operator* (qmatrix const &a, qmatrix const &b)
{
  qmatrix c;
  for (word i = 0; i < 3; ++i) {
    for(word j = 0; j < 3; ++j) {
      double total = 0.0;
      for (word k = 0; k < 3; ++k) 
	total += a.data[j * 3 + k] * b.data[k * 3 + i];
      c.data[j * 3 + i] = total;
    }
  }
  return c;
}

qmatrix operator- (qmatrix const &a, qmatrix const &b)
{
  qmatrix c;
  for (word i = 0; i < 9; ++i) 
    c.data[i] = a.data[i] - b.data[i];
  return c;
}

qmatrix operator+ (qmatrix const &a, qmatrix const &b)
{
  qmatrix c;
  for(word i = 0; i < 9; ++i)
    c.data[i] = a.data[i] + b.data[i];
  return c;
}

qmatrix transpose(qmatrix const &m)
{
  qmatrix p;
  p.data[0] = m.data[0];
  p.data[1] = m.data[3];
  p.data[2] = m.data[6];
  p.data[3] = m.data[1];
  p.data[4] = m.data[4];
  p.data[5] = m.data[7];
  p.data[6] = m.data[2];
  p.data[7] = m.data[5];
  p.data[8] = m.data[8];
  return p;
}

double magsqr(qmatrix const &m)
{
  double t = 0;
  for(word i = 0; i < 9; ++i)
    t += m.data[i] * m.data[i];
  return t;
}

qmatrix inverse (qmatrix const &m)
{
  static double A[9], B[9], WORK[20];
  static long JPVT[3];

  for (word i = 0; i < 3; ++i) 
    JPVT[i] = 0;
  A[0] = m.data[0];
  A[1] = m.data[3];
  A[2] = m.data[6];
  A[3] = m.data[1];
  A[4] = m.data[4];
  A[5] = m.data[7];
  A[6] = m.data[2];
  A[7] = m.data[5];
  A[8] = m.data[8];
  B[0] = 1.0;
  B[1] = 0.0;
  B[2] = 0.0;
  B[3] = 0.0;
  B[4] = 1.0;
  B[5] = 0.0;
  B[6] = 0.0;
  B[7] = 0.0;
  B[8] = 1.0;
  
  long N = 3;
  double RCOND = 1e-10;
  
  long RANK, INFO;
  gelsx (&N, &N, &N, A, &N, B, &N, JPVT, &RCOND, &RANK, WORK, &INFO);

  qmatrix p;
  p.data[0] = B[0];
  p.data[3] = B[1];
  p.data[6] = B[2];
  p.data[1] = B[3];
  p.data[4] = B[4];
  p.data[7] = B[5];
  p.data[2] = B[6];
  p.data[5] = B[7];
  p.data[8] = B[8];
  
  return p;
}

// Given three points in space, and the coordinates of a point in the
// frame defined by those points, figure out what the cartesian
// co-ordinates of that point might be. In general there will be two
// options, one on either side of the plane defined by the three
// points, if the points are colinear then the question is not
// properly stated.

void locate (pointset *ps, point *p, pointset *r)
{
  static qmatrix P, A, B, E, P1, P2, P3, M, X;
  double a, b, c, h;

  P1.vect (ps->item[0].coord[0], ps->item[0].coord[1], ps->item[0].coord[2]);
  P2.vect (ps->item[1].coord[0], ps->item[1].coord[1], ps->item[1].coord[2]);
  P3.vect (ps->item[2].coord[0], ps->item[2].coord[1], ps->item[2].coord[2]);

  B.vect (p->coord[0] * p->coord[0], p->coord[1] * p->coord[1], 
	  p->coord[2] * p->coord[2]);

  M.vect (magsqr (P1), magsqr (P2), magsqr (P3));
  B = B - M;

  P.data[0] = ps->item[0].coord[0];
  P.data[1] = ps->item[0].coord[1];
  P.data[2] = ps->item[0].coord[2];
  P.data[3] = ps->item[1].coord[0];
  P.data[4] = ps->item[1].coord[1];
  P.data[5] = ps->item[1].coord[2];
  P.data[6] = ps->item[2].coord[0];
  P.data[7] = ps->item[2].coord[1];
  P.data[8] = ps->item[2].coord[2];

  E.vect (1.0, 1.0, 1.0);

  A = inverse (-2.0 * P);

  // (-2.0 * P).put("P matrix\n","");
  // A.put("A matrix = \n","");
  // (transpose(A) * A).put("A^T * A matrix\n","");

  M = transpose (A) * A;

  a = (transpose (E) * M * E) . data[0];
  b = -(1.0 + 2.0 * (transpose (B) * M * E) . data[0]);
  c = (transpose(B) * M * B) . data[0];

  double des = b*b -4.0 * a * c;
  //printf("des = %f\n",(float)des);

  des = des<0.0 ? 0 : sqrt(des);

  // note ... if des is large magnitude -ve then the result is
  // garbage, but this protects against the slight incursions of the
  // discrimanent into the negative realm, which is the expected
  // situation
  
  h = (-b + des)/(2.0*a);
  X = A * (B - h*E);

  r->item[0].coord[0] = X.data[0];
  r->item[0].coord[1] = X.data[3];
  r->item[0].coord[2] = X.data[6];
  
  h = (-b - des) / (2.0*a);
  X = A * (B - h * E);
  
  r->item[1].coord[0] = X.data[0];
  r->item[1].coord[1] = X.data[3];
  r->item[1].coord[2] = X.data[6];
  
  r->n = 2;

  return;
}
