// cluster.hxx: point clusters
//
// Copyright (C) 2000 Bruce I Mills <B.I.Mills@cs.cf.ac.uk>
// Copyright (C) 2001-2002 Frank C Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.

class dualgraph;

// ================== Heap class needed for sorting ==================

// This is an implimentation of a priority queue 

class heap
{
public:  

  word n, n_max;
  numeric *key;
  word *item;

  heap (word l = 0) {
    if (l) {
      n_max = l;
      key = new numeric[l];
      item = new word[l];
    } else {
      n_max = 0;
      key = NULL;
      item = NULL;
    }
    n = 0;
  }
  heap (dualgraph const &g, pointset const &ps);

  ~heap () {
    if (key)
      delete[] key;
    if (item)
      delete[] item;
  }

  void push (numeric k, word p);
  word grab ();
  void empty ();
  void put (FILE *f = stdout) const;
  
private:

  word left_child (word l) {
    return (l << 1) + 1;
  }
  word right_child (word l) {
    return (l << 1) + 2;
  }
  word parent (word l) {
    return (l == 0) ? 0 : ((l - 1) >> 1);
  }

  void up ();
  void down ();
};

// ================= partition class ============================

// This class is a generic partition on the set of integers
// from 0 up to n-1. The idea is that it can be used to partition
// any indexed object, by partitioning the indices.
// However this partition also keeps track of some information
// as it goes along, such as how many nodes and links in each class
// and how many classes. The number of classes and the number of
// nodes in a class is clearly defined. By the number of links 
// what is meant is the number of merges that has been done to
// produce the given class. This information is specifically
// designed to help in examination of connected components of
// graphs.

class partition
{
public:

  partition (word k);

  ~partition () {
    delete[] parent;
    delete[] depth;
    delete[] links;
    delete[] nodes;
  }


  void merge(word a, word b); // merge two partitions
  word transitive() const {
    return incomplete == 0;
  }
  word singular() const {
    return classes == 1;
  }
  pointset *factor (pointset *p) const;
  dualgraph *factor (dualgraph *dg) const;

  word put (FILE *f = stdout, char *s1 = NULL, char *s2 = NULL);

private:

  word n;
  word *parent;     // the parent of each node in its class tree
  word *depth;      // the depth of the tree (root node only)
  word *links;      // number of links in the tree (root node only)
  word *nodes;      // number of nodes in the tree (root node only)
  word incomplete;  // the number of incomplete classes
  word classes;     // the number of classes

  word root(word a) const; // find the root of the class tree
};
