/* config.h: Master configuration header file.
 * 
 * Copyright (C) 2001-2002 Frank Langbein <frank@langbein.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* NOTE: This header file should be included first by all source files, but
 * not by any of the other header files. 
 *
 * There is no need to edit this file for changing the configuration. This
 * should always be done by the autoconf script configure and the header
 * file generated by that script.
 *
 * Note that there are some configuration options at the end of the file.
 * They should eventually be moved somewhere else or eliminated.
 */

#ifndef CONFIG_H
#define CONFIG_H 1

/* Include the configuration parameters from configure/autoconf first. */
#ifdef HAVE_CONFIG_H
#include "c-auto.h"
#endif /* HAVE_CONFIG_H */

/* If we are using glibc this enables some useful extensions, otherwise
 * it should have no effect. */
#define _GNU_SOURCE 1

/* Standard C header files */

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */

#ifdef HAVE_MATH_H
#include <math.h>
#endif /* HAVE_MATH_H */

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif /* HAVE_CTYPE_H */

#ifdef __cplusplus
/* We only need this in the C++ code and that way we avoid some warnings. */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#endif /* __cplusplus */

/* Standard C++ header files */

#ifdef __cplusplus

#ifdef HAVE_IOSTREAM_H
#include <iostream.h>
#endif /* HAVE_IOSTRAM_H */

#ifdef HAVE_FSTREAM_H
#include <fstream.h>
#endif /* HAVE_FSTREAM_H */

#endif /* __cplusplus */

/* Compiler characteristics. */

/* The NULL pointer should be defined on most systems. But there are
 * still some which don't provide it. */
#ifndef NULL
#ifdef __cplusplus
#define NULL 0
#else /* __cplusplus */
#define NULL (void *)0
#endif /* !__cplusplus */
#endif /* !NULL */

/* Define const to be empty if it is not already defined or provided
 * by the compiler. */
#if !defined(__STDC__) || !__STDC__
/* This is a separate conditional since some standard C systems reject
 * `defined (const)'. */
#ifndef const
#define const
#endif /* !const */
#endif /* !defined(__STDC__) || !__STDC__ */

/* Precision used for numerical computations (do not change) */
typedef double numeric;

/* Byte and word as unsigned numbers to avoid trouble with indices and
 * other arithmetic. */
typedef unsigned char byte;
typedef unsigned int word;

/* Epsilon for numerical precision. Any number smaller than this is
 * considered to be zero. This number should be sufficiently small
 * to indicate the numerical noise level, but is not intended to be
 * used for `approximate' results. This can be changed by a command 
 * line option. */
#define DEFAULT_EPSILON (0.00000000000001)
/* Current epsilon. */
extern numeric epsilon;

/* Program name */
extern char *progname;

#endif /* !CONFIG_H */
