/* lapack.h: Lapack definitions and declarations.
 * 
 * Copyright (C) 2001-2002 Frank C Langbein <frank@langbein.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: lapack.h 1 2005-08-10 23:32:56Z frank $
 */

#ifndef LAPACK_H
#define LAPACK_H 1

/* Required for lapack / f2c */

#ifdef HAVE_F2C_H
/* f2c.h is required for lapack as it's a Fortran 77 library and we are 
 * using it with C. On some systems the f2c.h header might not be present.
 * There we can only hope that it works withoutit.
 *
 * We treat the f2c.h as C code. If this file is included by a C++ compiler
 * __cplusplus will be undefined during the inclusion of f2c.h to avoid
 * warnings about `...' without first agument. To remember this we define
 * CPLUSPLUS.
 *
 * We also undefine a few things set by f2c.h which interfere with out
 * own code.
 */
#ifdef __cplusplus
extern "C" {
#undef __cplusplus
#define CPLUSPLUS 1
#endif /* __cplusplus */
#include <f2c.h>
#ifdef max
#undef max
#endif /* max */
#ifdef min
#undef min
#endif /* min */
#ifdef abs
#undef abs
#endif /* abs */
#ifdef CPLUSPLUS
#define __cplusplus
#undef CPLUSPLUS
}
#endif /* CPLUSPLUS */
#endif /* HAVE_F2C_H */

/* On some systems the symbols in the lapack library have an
 * underscore suffix. If we are on such a system (dectected by the
 * autoconf script) we define the symbols without underscore here.
 */
#ifdef F77LAPACK_UNDERSCORE
#define dgelsx dgelsx_
#endif /* F77LAPACK_UNDERSCORE */

#ifdef	__cplusplus
extern "C" {
#endif /* __cplusplus */


#define INV dgelsx
void dgelsx (long *M,        // rows in matrix A
	     long *N,        // cols in matrix A
	     long *NRHS,     // cols in matrix B
	     double *A,      // the matrix A
	     long *LDA,      // (essentialy the rows in matrix A)
	     double *B,      // the matrix B
	     long *LDB,      // (essentially the rows in matrix B)
	     long *JPVT,     // integer array
	     double *RCOND,  // a small number
	     long *RANK,
	     double *WORK,   // work space in our cae 3*N
	     long *INFO);

#define gelsx dgelsx

#ifdef	__cplusplus
}
#endif /* __cplusplus */

#endif /* LAPACK_H */
