// xfig.cc - xfig record handling routines
//
// Copyright (C) 2000 Bruce I Mills <B.I.Mills@cs.cf.ac.uk>
// Copyright (C) 2001-2002 Frank C Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.

#include "config.h"
#include "xfig.hxx"
#include "points.hxx"

void skip (FILE *f) {
  int c;
  while ((c = fgetc (f)) != EOF) {
    if (!isblank (c) && c != '\n') {
      ungetc (c, f);
      return;
    }
  }
  return;
}

// write a standard header for an xfig file

word xfig_header(FILE *f, char *ver)
{
  word count = 0;
  if (!strcmp (ver, "FIG 3.2")) {
    count += fprintf (f, "#FIG 3.2   \n");
    count += fprintf (f, "Landscape  \n");
    count += fprintf (f, "Center     \n");
    count += fprintf (f, "Metric     \n");
    count += fprintf (f, "A4         \n");
    count += fprintf (f, "100.00     \n");
    count += fprintf (f, "Single     \n");
    count += fprintf (f, "-2         \n");   
    count += fprintf (f, "1200 2     \n");
  } else {
    if(!strcmp (ver, "FIG 3.1")) {
      count += fprintf (f, "#FIG 3.1\n");
      count += fprintf (f, "Landscape\n");
      count += fprintf (f, "Center\n");
      count += fprintf (f, "Inches\n");
      count += fprintf (f, "1200 2\n");
    } else {
      count += fprintf (f, "#FIG 3.1\n");
      count += fprintf (f, "Landscape\n");
      count += fprintf (f, "Center\n");
      count += fprintf (f, "Inches\n");
      count += fprintf (f, "1200 2\n");
    }
  }
  return count;
}

// xfig point record class

void xfig_point_record::standard (point p)
{
  field01 = 3;
  field02 = 0;
  field03 = 1;
  field04 = 0;
  field05 = 7;
  field06 = 100;
  field07 = 0;
  field08 = -1;
  field09 = 0.000;
  field10 = 1; 
  field11 = 0.0000;
  
  field12 =  static_cast<int> (p.coord[0]);
  field13 =  static_cast<int> (p.coord[1]);
  field14 =  64;
  field15 =  64;
  field16 =  static_cast<int> (p.coord[0]);
  field17 =  static_cast<int> (p.coord[1]);
  field18 =  static_cast<int> (p.coord[0] + 45);
  field19 =  static_cast<int> (p.coord[1] + 45);

  return;
}

void xfig_point_record::get(FILE *f)
{
  skip (f);
  fscanf (f,"%d",&field01);
  skip (f);
  fscanf (f,"%d",&field02);
  skip (f);
  fscanf (f,"%d",&field03);
  skip (f);
  fscanf (f,"%d",&field04);
  skip (f);
  fscanf (f,"%d",&field05);
  skip (f);
  fscanf (f,"%d",&field06);
  skip (f);
  fscanf (f,"%d",&field07);
  skip (f);
  fscanf (f,"%d",&field08);
  skip (f);
  fscanf (f,"%f",&field09);
  skip (f);
  fscanf (f,"%d",&field10);
  skip (f);
  fscanf (f,"%f",&field11);
  skip (f);
  fscanf (f,"%d",&field12);   
  skip (f);
  fscanf (f,"%d",&field13);   
  skip (f);
  fscanf (f,"%d",&field14);   
  skip (f);
  fscanf (f,"%d",&field15);
  skip (f);
  fscanf (f,"%d",&field16);
  skip (f);
  fscanf (f,"%d",&field17);
  skip (f);
  fscanf (f,"%d",&field18);
  skip (f);
  fscanf (f,"%d",&field19);
  return;
}

word xfig_point_record::put(FILE *f)
{
  word count = fprintf(f,"1 ");
  count += fprintf(f,"%d "   ,field01);
  count += fprintf(f,"%d "   ,field02);
  count += fprintf(f,"%d "   ,field03);
  count += fprintf(f,"%d "   ,field04);
  count += fprintf(f,"%d "   ,field05);
  count += fprintf(f,"%d "   ,field06);
  count += fprintf(f,"%d "   ,field07);
  count += fprintf(f,"%d "   ,field08);
  count += fprintf(f,"%5.3f ",field09);
  count += fprintf(f,"%d "   ,field10);
  count += fprintf(f,"%6.4f ",field11);
  count += fprintf(f,"%d "   ,field12);   
  count += fprintf(f,"%d "   ,field13);   
  count += fprintf(f,"%d "   ,field14);   
  count += fprintf(f,"%d "   ,field15);
  count += fprintf(f,"%d "   ,field16);
  count += fprintf(f,"%d "   ,field17);
  count += fprintf(f,"%d "   ,field18);
  count += fprintf(f,"%d "   ,field19);
  count += fprintf(f,"\n");
  return count;
}

// xfig line record class

void xfig_line_record::standard(point p1, point p2)
{
  field01  = 1    ; // (some sort of style file)
  field02  = 0    ; // line style (0=solid 2=dashed)
  field03  = 1    ; // line thickness
  field04  = 0    ; // pen colour
  field05  = 7    ; // fill colour
  field06  = 100  ; // xfig layer depth
  field07  = 0    ; // (doesnt seem to do anything)
  field08  = -1   ; // fill style (-1=none)
  field09  = 0.000; // gap to solid ratio
  field10  = 0    ; // join style (0=miter 1=round)
  field11  = 0    ; // cap style (0=butt 1=round)
  field12  = -1   ; // fill intensity
  field13  = 0    ; // forward arrow flag
  field14  = 0    ; // back arrow flag
  field15  = 2    ; // number of points
  field16  = static_cast<int> (p1.coord[0]); // x coord of point 1
  field17  = static_cast<int> (p1.coord[1]); // y coord of point 1
  field18  = static_cast<int> (p2.coord[0]); // x coord of point 2
  field19  = static_cast<int> (p2.coord[1]); // y coord of point 2
  return;
}

word xfig_line_record::put(FILE *f)
{
  word count = fprintf(f,"2 ");             // xfig polyline type number
  count += fprintf(f,"%d ",   field01); // (some sort of style file)
  count += fprintf(f,"%d ",   field02); // line style (0=solid 2=dashed)
  count += fprintf(f,"%d ",   field03); // line thickness
  count += fprintf(f,"%d ",   field04); // pen colour
  count += fprintf(f,"%d ",   field05); // fill colour
  count += fprintf(f,"%d ",   field06); // xfig layer depth
  count += fprintf(f,"%d ",   field07); // (doesnt seem to do anything)
  count += fprintf(f,"%d ",   field08); // fill style (-1=none)
  count += fprintf(f,"%5.3f ",field09); // gap to solid ratio
  count += fprintf(f,"%d ",   field10); // join style (0=miter 1=round)
  count += fprintf(f,"%d ",   field11); // cap style (0=butt 1=round)
  count += fprintf(f,"%d ",   field12); // fill intensity
  count += fprintf(f,"%d ",   field13); // forward arrow flag
  count += fprintf(f,"%d ",   field14); // back arrow flag
  count += fprintf(f,"%d ",   field15); // number of points
  count += fprintf(f,"\n");
  count += fprintf(f,"%d ", field16);
  count += fprintf(f,"%d ", field17);
  count += fprintf(f,"%d ", field18);
  count += fprintf(f,"%d ", field19);
  count += fprintf(f,"\n");
  return count;
}

void xfig_line_record::get(FILE *f)
{
  skip (f);
  fscanf(f,"%d",&field01);
  skip (f);
  fscanf(f,"%d",&field02);
  skip (f);
  fscanf(f,"%d",&field03);
  skip (f);
  fscanf(f,"%d",&field04);
  skip (f);
  fscanf(f,"%d",&field05);
  skip (f);
  fscanf(f,"%d",&field06);
  skip (f);
  fscanf(f,"%d",&field07);
  skip (f);
  fscanf(f,"%d",&field08);
  
  skip (f);
  fscanf(f,"%f",&field09);
  
  skip (f);
  fscanf(f,"%d",&field10);
  skip (f);
  fscanf(f,"%d",&field11);
  skip (f);
  fscanf(f,"%d",&field12);
  skip (f);
  fscanf(f,"%d",&field13);
  skip (f);
  fscanf(f,"%d",&field14);
  skip (f);
  fscanf(f,"%d",&field15);
  
  skip (f);
  fscanf(f,"%d",&field16);
  skip (f);
  fscanf(f,"%d",&field17);  
  skip (f);
  fscanf(f,"%d",&field18);  
  skip (f);
  fscanf(f,"%d",&field19);

  return;
}


// xfig text record class

word xfig_text_record::put(FILE *f)
{
  word count = fprintf(f,"4 ");
  count += fprintf(f,"%d ",field01);
  count += fprintf(f,"%d ",field02);
  count += fprintf(f,"%d ",field03);
  count += fprintf(f,"%d ",field04);
  count += fprintf(f,"%d ",field05);
  count += fprintf(f,"%d ",field06);
  count += fprintf(f,"%d ",static_cast<int> (field07));
  count += fprintf(f,"%d ",field08);
  count += fprintf(f,"%d ",field09);
  count += fprintf(f,"%d ",field10);
  count += fprintf(f,"%d ",field11);
  count += fprintf(f,"%d ",field12);
  count += fprintf(f,"%s\\001",field13);
  count += fprintf(f,"\n");
  return count;
}

void xfig_text_record::get(FILE *f)
{
  return;
}

void xfig_text_record::standard(point p, char *s)
{
  field01 = 0;
  field02 = 0;
  field03 = 100;
  field04 = 0;
  field05 = 0;
  field06 = 12;
  field07 = 0.0000;
  field08 = 4;
  field09 = 135;
  field10 = 270;
  field11 = static_cast<int> (p.coord[0]);
  field12 = static_cast<int> (p.coord[1]);
  strcpy(field13,s);
  return;
}
